package com.xmlunit.samples;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.sql.Clob;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.custommonkey.xmlunit.DetailedDiff;
import org.custommonkey.xmlunit.Diff;
import org.custommonkey.xmlunit.Difference;
import org.custommonkey.xmlunit.DifferenceConstants;
import org.custommonkey.xmlunit.DifferenceListener;
import org.custommonkey.xmlunit.XMLUnit;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import model.ReportModel;
import model.XMLDifference;

public class XMLCompare {
	public XMLCompare() {
	}

	static int depthOfXML = 0;

	public static List<String> generateListFromFile(String ignoreTagFilePath) {
		final List<String> ignoreTagList = new ArrayList<String>();
		try {

			// Open the file that is the first
			// command line parameter
			FileInputStream fstream = new FileInputStream(ignoreTagFilePath);
			// Get the object of DataInputStream
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;
			// Read File Line By Line
			while ((strLine = br.readLine()) != null) {
				// Print the content on the console
				// System.out.println(strLine);
				ignoreTagList.add(strLine.trim());
			}
			// Close the input stream
			in.close();
		} catch (Exception e) {// Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}
		return ignoreTagList;
	}

	public static int getTagCount(String XMLpath) {
		int level = 1;
		// System.out.println(elements.getNodeName() + "[" + level + "]");
		depthOfXML = 1;
		try {
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(XMLpath);

			Element elements = doc.getDocumentElement();

			NodeList nodeList = elements.getChildNodes();
			printNode(nodeList, level);

			// System.out.println("Total Tag Count : " + depthOfXML);

		} catch (ParserConfigurationException pce) {
			System.out.println("pce: " + pce);
			depthOfXML = 0;
		} catch (IOException ioe) {
			System.out.println("pce: " + ioe);
			depthOfXML = 0;
		} catch (SAXException sae) {
			System.out.println("pce: " + sae);
			depthOfXML = 0;
		} catch (Exception e) {
			System.out.println("pce: " + e);
			depthOfXML = 0;
		}
		return depthOfXML;
	}

	private static void printNode(NodeList nodeList, int level) {
		level++;
		if (nodeList != null && nodeList.getLength() > 0) {

			for (int i = 0; i < nodeList.getLength(); i++) {

				Node node = nodeList.item(i);
				if (node.getNodeType() == Node.ELEMENT_NODE) {

					// System.out.println(node.getNodeName() + "[" + level +
					// "]");
					printNode(node.getChildNodes(), level);
					// if (level > depthOfXML) {
					depthOfXML++;
					// }
				}

			}

		}

	}

	public static List<XMLDifference> compareXML(String pathName1, String pathName2, String ignoreTagFilePath) {
		List<XMLDifference> retValue = new ArrayList<XMLDifference>();
		final List<String> ignoreTagList = generateListFromFile(ignoreTagFilePath);
		String sourceFile = null;
		String targetFile = null;
		try {
			FileInputStream fSstream = new FileInputStream(pathName1);
			// Get the object of DataInputStream
			DataInputStream inS = new DataInputStream(fSstream);
			BufferedReader brS = new BufferedReader(new InputStreamReader(inS));
			sourceFile = brS.readLine();
			brS.close();
			FileInputStream fTstream = new FileInputStream(pathName2);
			// Get the object of DataInputStream
			DataInputStream inT = new DataInputStream(fTstream);
			BufferedReader brT = new BufferedReader(new InputStreamReader(inT));
			targetFile = brT.readLine();
			brT.close();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			// e1.printStackTrace();
		}

		if (sourceFile == null && targetFile == null) {
			retValue.add(
					new XMLDifference(null, null, null, null, "Source and Target files are Empty or Invalid", null));
			return retValue;
		} else if (sourceFile == null) {
			retValue.add(new XMLDifference(null, null, null, null, "Source file is Empty or Invalid", null));
			return retValue;
		} else if (targetFile == null) {
			retValue.add(new XMLDifference(null, null, null, null, "Target file is Empty or Invalid", null));
			return retValue;
		}

		FileReader fr1 = null;
		FileReader fr2 = null;
		try {
			fr1 = new FileReader(pathName1);
			fr2 = new FileReader(pathName2);

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		try {
			XMLUnit.setIgnoreWhitespace(true);
			XMLUnit.setIgnoreComments(true);
			XMLUnit.setIgnoreDiffBetweenTextAndCDATA(true);
			XMLUnit.setNormalizeWhitespace(true);
			XMLUnit.setIgnoreAttributeOrder(true);

			Diff diff = null;
			diff = new Diff(fr1, fr2);

			diff.overrideDifferenceListener(new DifferenceListener() {
				public int differenceFound(Difference diff) {
					final String xpathLocation = diff.getTestNodeDetail().getXpathLocation();
					final String sourceXpathLocation = diff.getControlNodeDetail().getXpathLocation();
					// System.out.println(diff.getId() +" === "
					// +DifferenceConstants.ATTR_SEQUENCE_ID+"=="
					// +diff.getDescription());
					if (diff.getId() == DifferenceConstants.NAMESPACE_PREFIX_ID) {
						return DifferenceListener.RETURN_IGNORE_DIFFERENCE_NODES_IDENTICAL;
					} else if (diff.getId() == 19 || diff.getId() == 20
							|| diff.getId() == DifferenceConstants.TEXT_VALUE_ID) {
						return DifferenceListener.RETURN_IGNORE_DIFFERENCE_NODES_IDENTICAL;
					} else if (ignoreTagList.contains(xpathLocation)) {
						// System.out.println("Ignore List Path : " +
						// xpathLocation);
						return DifferenceListener.RETURN_IGNORE_DIFFERENCE_NODES_IDENTICAL;
					} else if (ignoreTagList.contains(sourceXpathLocation)) {
						// System.out.println("Ignore List Path " +
						// sourceXpathLocation);
						return DifferenceListener.RETURN_IGNORE_DIFFERENCE_NODES_IDENTICAL;
					}

					return DifferenceListener.RETURN_ACCEPT_DIFFERENCE; /**
																		 * no
																		 * objection,
																		 * mark
																		 * it as
																		 * a
																		 * valid
																		 * difference
																		 */
				}

				public void skippedComparison(Node arg0, Node arg1) {
				}
			});
			// System.out.println("Similar? " + diff.similar());
			// System.out.println("Identical? " + diff.identical());

			DetailedDiff detDiff = new DetailedDiff(diff);

			List<Object> differences = detDiff.getAllDifferences();
			for (Object object : differences) {
				Difference difference = (Difference) object;
				String sourceNode = null;
				String sourceNodeValue = null;
				String targetNodeValue = null;
				String description = null;
				String ignorePath = null;
				String targetNode = null;
				try {
					sourceNode = difference.getControlNodeDetail().getNode().getParentNode().getNodeName();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					sourceNode = "NA";
				}
				try {
					targetNode = difference.getTestNodeDetail().getNode().getParentNode().getNodeName();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					sourceNode = "NA";
				}
				try {
					sourceNodeValue = difference.getControlNodeDetail().getValue() == "null" ? "NA"
							: difference.getControlNodeDetail().getValue();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					sourceNode = "NA";
				}
				try {
					targetNodeValue = difference.getTestNodeDetail().getValue() == "null" ? "NA"
							: difference.getTestNodeDetail().getValue();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					sourceNode = "NA";
				}
				description = difference.getDescription();

				// System.out.println("description : " + description);
				ignorePath = difference.getTestNodeDetail().getXpathLocation() != null
						? difference.getTestNodeDetail().getXpathLocation()
						: difference.getControlNodeDetail().getXpathLocation();

				description = difference.toString();

				if (sourceNode != null || targetNode != null || sourceNodeValue != null || targetNodeValue != null
						|| 1 == 1) {
					retValue.add(new XMLDifference(sourceNode, targetNode, sourceNodeValue, targetNodeValue,
							description, ignorePath));
				}
			}

		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return retValue;
	}

	public static String validateWSDL(String wsdlFilePath, String payLoadFilePath) {
		String retVal = null;
		try {
			FileInputStream wsdlFis = new FileInputStream(wsdlFilePath);
			byte[] wsdlByte = new byte[wsdlFis.available()];
			wsdlFis.read(wsdlByte);
			String wsdlStr = new String(wsdlByte);

			FileInputStream payLoadFis = new FileInputStream(payLoadFilePath);
			byte[] payLoadByte = new byte[payLoadFis.available()];
			payLoadFis.read(payLoadByte);
			String payLoadStr = new String(payLoadByte);

			// retVal = SOAPValidatorAPI.validate(wsdlStr, payLoadStr);
			System.out.println("retStr : " + retVal);

			wsdlFis.close();
			payLoadFis.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
		}

		return "SUCCESS:VALID";
	}

	public static void main(String[] args) {
		try {
			String legacyXMLPath = "D:/Users/jyotikumar.s/Documents/Java Projects/Run6_CustomerService_getSubscriberInfo_LegacyRes.xml";
			String migratedXMLPath = "D:/Users/jyotikumar.s/Documents/Java Projects/Run6_CustomerService_getSubscriberInfo_MigratedResp.xml";
			String ignoreTagFilePath = "D:/Users/jyotikumar.s/Documents/Java Projects/IgnoreTagList.txt";
			String serviceName = "Test";
			String reportPath = "D:\\";

			compareXMLAndGenerateReport(serviceName, legacyXMLPath, migratedXMLPath, ignoreTagFilePath, reportPath);

			// String wsdlFilePath = "D:/Users/jyotikumar.s/Documents/Java
			// Projects/IgnoreTagList.txt";
			// String payLoadFilePath = "D:/Users/jyotikumar.s/Documents/Java
			// Projects/IgnoreTagList.txt";
			// String retWSDLVal = null;
			//
			// try {
			// retWSDLVal = validateWSDL(wsdlFilePath, payLoadFilePath);
			// } catch (Exception e) {
			// // TODO Auto-generated catch block
			// // e.printStackTrace();
			// }
			// System.out.println(retWSDLVal);
			/*
			 * if (retWSDLVal.equalsIgnoreCase("SUCCESS:VALID")) { int
			 * sourceTagCount = getTagCount(expectedXml); int targetTagCount =
			 * getTagCount(generatedXml);
			 * 
			 * System.out.println("Total Tag count in Source XML : " +
			 * sourceTagCount); System.out.println(
			 * "Total Tag count in Target XML : " + targetTagCount);
			 * 
			 * List<XMLDifference> outValList = compareXML(expectedXml,
			 * generatedXml, ignoreTagFilePath); int count = 1; boolean
			 * validFile = true; System.out.println(
			 * "********* Response Difference Details**************"); for
			 * (XMLDifference outList : outValList) { System.out.println(
			 * "SI No. : " + count++); System.out.println("Source Node : [" +
			 * outList.getSourceNode() + "] Value : [" +
			 * outList.getSourceNodeValue() + "]"); System.out.println(
			 * "Target Node : [" + outList.getTargetNode() + "] Value : [" +
			 * outList.getTargetNodeValue() + "]"); System.out.println(
			 * "Difference Description : " + outList.getDescription());
			 * System.out.println("Node Path : " + outList.getIgnorePath());
			 * System.out.println(
			 * "-----------------------------------------------"); if (count ==
			 * 1 && outList.getIgnorePath() == null) { validFile = false; } }
			 * 
			 * if (sourceTagCount == targetTagCount && count == 1) {
			 * System.out.println(
			 * "Success : The Source and Target XML's are Identical."); } else
			 * if (sourceTagCount != targetTagCount && count == 1) {
			 * System.out.println(
			 * "Warning : The Source and Target XML's are Identical by Ignoring certain elements."
			 * ); }
			 * 
			 * System.out.println("************ End Of Report ***********");
			 * 
			 * String reportPath = "D:/Reports/";
			 * compareXMLAndGenerateReport(expectedXml, generatedXml,
			 * reportPath);
			 * 
			 * 
			 * }
			 */
			// String CSVFilepath =
			// "D:\\Reports\\Comparison_XML_File_Format.csv";
			// System.out.println("No. of Argument Passesd " + args.length);
			// if (args.length > 0) {
			// String CSVFilepath = args[0];
			// compareXMLFromCSVAndGenerateReport(CSVFilepath);
			// } else {
			// System.out.println("No. of Argument Passesd " + args.length);
			// System.out.println(
			// "Please pass the Comparison_XML_List.csv file path along with
			// file name.(eg. D:\\Comparison_XML_List.csv)");
			// }

		} catch (Exception ex) {
			// ex.printStackTrace();
		}
	}

	public static void compareXMLAndGenerateReport(String serviceName, String sourceXMLPath, String targetXmlPath,
			String ignoreTagFilePath, String reportPath) {

		try {
			List<XMLDifference> outValList = compareXML(sourceXMLPath, targetXmlPath, ignoreTagFilePath);

			DateFormat dateFormat = new SimpleDateFormat("yyyy_MM_dd_HHmmss");
			Date date = new Date();
			reportPath = createDirectory(reportPath, null) + "\\" + serviceName + "_Comparison_Report_"
					+ dateFormat.format(date) + ".html";

			generateReport(reportPath, outValList, serviceName, sourceXMLPath, targetXmlPath);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private static Map<Integer, ReportModel> compareXMLReport(String pathName1, String pathName2,
			String ignoreTagFilePath) {
		Map<Integer, ReportModel> diffReportMap = new HashMap<Integer, ReportModel>();
		FileReader fr1 = null;
		FileReader fr2 = null;
		try {
			fr1 = new FileReader(pathName1);
			fr2 = new FileReader(pathName2);

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		try {
			XMLUnit.setIgnoreWhitespace(true);
			XMLUnit.setIgnoreComments(true);
			XMLUnit.setIgnoreDiffBetweenTextAndCDATA(true);
			XMLUnit.setNormalizeWhitespace(true);
			XMLUnit.setIgnoreAttributeOrder(true);

			Diff diff = null;
			diff = new Diff(fr1, fr2);

			diff.overrideDifferenceListener(new DifferenceListener() {
				public int differenceFound(Difference diff) {

					if (diff.getId() == DifferenceConstants.NAMESPACE_PREFIX_ID
							|| diff.getId() == DifferenceConstants.TEXT_VALUE_ID) {
						return DifferenceListener.RETURN_IGNORE_DIFFERENCE_NODES_IDENTICAL;
					}

					return DifferenceListener.RETURN_ACCEPT_DIFFERENCE; /**
																		 * no
																		 * objection,
																		 * mark
																		 * it as
																		 * a
																		 * valid
																		 * difference
																		 */
				}

				public void skippedComparison(Node arg0, Node arg1) {
				}
			});

			DetailedDiff detDiff = new DetailedDiff(diff);
			String description = null;
			Integer recCount = 0;
			ReportModel reportModelObj;
			List<XMLDifference> diffList;
			String sourceNode = "NA";
			String sourceNodeValue = "NA";
			String targetNodeValue = "NA";
			String ignorePath = "NA";
			String targetNode = "NA";
			Integer key = null;

			List<Object> differences = detDiff.getAllDifferences();
			for (Object object : differences) {
				Difference difference = (Difference) object;

				try {
					try {
						key = difference.getId();
						sourceNode = difference.getControlNodeDetail().getNode() != null
								? difference.getControlNodeDetail().getNode().getParentNode().getNodeName() : "NA";
						sourceNodeValue = difference.getControlNodeDetail() != null
								? difference.getControlNodeDetail().getValue() : "NA";
						targetNodeValue = difference.getTestNodeDetail() != null
								? difference.getTestNodeDetail().getValue() : "NA";
						description = difference.getDescription();
						ignorePath = difference.toString();

						targetNode = difference.getTestNodeDetail().getNode() != null
								? difference.getTestNodeDetail().getNode().getParentNode().getNodeName() : "NA";
					} catch (Exception e) {
						// TODO Auto-generated catch block
						// e.printStackTrace();
					}
					// System.out.println("Key : "+difference.getId()+"
					// Description : " +difference.getDescription() + " Count :
					// " + 1);

					if (diffReportMap.containsKey(key)) {
						System.out.println("Key : " + key + " Available. sourceNode = " + sourceNodeValue);
						reportModelObj = diffReportMap.get(key);
						reportModelObj.setCount(reportModelObj.getCount() + 1);
						diffList = reportModelObj.getRetValue();
						diffList.add(new XMLDifference(sourceNode, targetNode, sourceNodeValue, targetNodeValue,
								description, ignorePath));
					} else {
						System.out.println("Key : " + key + " Available. sourceNode = " + sourceNode);
						diffList = new ArrayList<XMLDifference>();
						diffList.add(new XMLDifference(sourceNode, targetNode, sourceNodeValue, targetNodeValue,
								description, ignorePath));
						reportModelObj = new ReportModel(++recCount, description, 1, diffList);
					}

					diffReportMap.put(difference.getId(), reportModelObj);

				} catch (Exception e) {
					// TODO Auto-generated catch block
					// e.printStackTrace();
				}
			}

		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return diffReportMap;
	}

	/**
	 * This method create a directory path if not exist in system.
	 * 
	 * @param Path
	 *            - Directory Path were response/request need to be placed.
	 * @return New Path.
	 */
	private static String createDirectory(String Path, String dir) {
		String newFilePath = null;
		if (dir != null) {
			newFilePath = Path + dir;
		} else {
			newFilePath = Path;
		}
		try {
			File file = new File(newFilePath);
			if (!file.exists()) {
				if (file.mkdir()) {
					System.out.println("Directory is created!");
				} else {
					System.out.println("Failed to create directory!");
				}

			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// System.out.println("newFilePath : " + newFilePath);
		return newFilePath;

	}

	private static void generateReport(String fileNameWithPath, Map<Integer, ReportModel> reportModelList) {
		// try {
		// File data = new File(fileNameWithPath);
		// FileWriter writer = new FileWriter(data);
		// StringBuffer strOut = new StringBuffer();
		//
		// strOut.append("<table style='table-layout: fixed;width: 100%;
		// background-color: #D6EAF8;'>\n");
		// strOut.append(
		// "<tr><th><h2 style='text-align: center;'><span style='color:
		// #1B4F72;'><strong><u>XML Comparison Detail
		// Report</u></strong></span></h2>");
		// reportModelList.forEach((k, v) -> {
		// List<XMLDifference> diffList = v.getRetValue();
		//
		// strOut.append(
		// "<table border=1; style='width: 100%; background-color:
		// #EBF5FB;'><caption>&nbsp;</caption>\n");
		// strOut.append("<tbody>\n");
		// strOut.append(
		// "<tr><th style='background-color: #2874A6;'>&nbsp;SI No.</th><th
		// colspan='3' style='background-color: #2874A6;'>Difference</th><th
		// style='background-color: #2874A6;'>Difference Count</th></tr>\n");
		// strOut.append("<tr>\n");
		// strOut.append("<td style='text-align: center;'>" + v.getId() +
		// "</td>\n");
		// strOut.append("<td colspan='3'>The " + v.getDesc() + " not
		// matches.</td>\n");
		// strOut.append("<td style='text-align: center;'>" + v.getCount() +
		// "</td>\n");
		// strOut.append("</tr>\n");
		// strOut.append("<tr><th colspan='5'>Details</th></tr>\n");
		// strOut.append(
		// "<tr><th style='background-color: #2874A6;'>Source Node</th><th
		// style='background-color: #2874A6;'>Source Node Value</th><th
		// style='background-color: #2874A6;'>Target Node</th><th
		// style='background-color: #2874A6;'>Target Node Value</th><th
		// style='background-color: #2874A6;'>Description</th></tr>\n");
		// diffList.forEach(item -> {
		// strOut.append("<tr>\n");
		// strOut.append("<td style='width: 12%; word-wrap: break-word;
		// word-break: break-all; '>"
		// + item.getSourceNode() + "</td>\n");
		// strOut.append(
		// "<td style='width: 12%; text-align: center; word-wrap: break-word;
		// word-break: break-all; '>"
		// + item.getSourceNodeValue() + "</td>\n");
		// strOut.append("<td style='width: 12%; word-wrap: break-word;
		// word-break: break-all; '>"
		// + item.getTargetNode() + "</td>\n");
		// strOut.append(
		// "<td style='width: 12%; text-align: center; word-wrap: break-word;
		// word-break: break-all; '>"
		// + item.getTargetNodeValue() + "</td>\n");
		// strOut.append("<td style='width: 52%; word-wrap: break-word;
		// word-break: break-all; '>"
		// + item.getIgnorePath() + "</td>\n");
		// strOut.append("</tr>\n");
		// });
		// strOut.append("</tbody>\n");
		// strOut.append("</table>\n");
		// });
		// strOut.append("</th></tr>");
		// strOut.append("</table>\n");
		//
		// String clobStr = strOut.toString();
		//
		// writer.write(clobStr);
		// writer.close();
		// } catch (Exception e) {
		// e.printStackTrace();
		// }

	}

	public static List<XMLDifference> compareXMLFromCSVAndGenerateReport(String CSVFilepath) {
		// System.out.println("CSVFilepath : " + CSVFilepath);
		int lineCount = 0;
		String serviceName = null;
		String sourceXML = null;
		String targetXML = null;
		String reportPath = null;
		String ignoreTagFilePath = null;

		List<XMLDifference> diffReportMap = new ArrayList<XMLDifference>();
		List<String> list = new ArrayList<>();

		try (Stream<String> stream = Files.lines(Paths.get(CSVFilepath))) {
			// Skip Header from service configuration file and populated all
			// rows to List.
			// list = stream.skip(1).collect(Collectors.toList());
		} catch (IOException e) {
			System.out.println("Configuration File Not found in mentioned path(" + CSVFilepath + ").");
			return diffReportMap;
		}

		// Process service configuration file record one by one
		for (String line : list) {

			try {
				++lineCount;
				// System.out.println("line :" + lineCount);
				String[] data = line.split(",");
				// System.out.println("line : " + line);
				// Log Error if all mandatory inputs are not available in
				// service configuration file.
				if (data.length != 4) {
					System.out.println(
							"The Configuration file must contains 3 parameter for each row, please verify line : "
									+ lineCount + ".");
					continue;
				}

				serviceName = data[0].trim();
				sourceXML = (data[1] != null ? data[1].trim() : null);
				targetXML = (data[2] != null ? data[2].trim() : null);
				ignoreTagFilePath = (data[3] != null ? data[3].trim() : null);
				reportPath = sourceXML.substring(0, sourceXML.lastIndexOf("\\"));

				// System.out.println("SI No. - " + lineCount);
				// System.out.println("----- Input Details ----");
				// System.out.println("serviceName : " + serviceName);
				// System.out.println("sourceXML : " + sourceXML);
				// System.out.println("targetXml : " + targetXML);
				// System.out.println("reportPath : " + reportPath);
				// System.out.println("ignoreTagFilePath : " +
				// ignoreTagFilePath);
				// System.out.println("................................");

				diffReportMap = compareXML(sourceXML, targetXML, ignoreTagFilePath);
				// int count = 1;
				//
				// System.out.println("********* Response Difference
				// Details**************");
				// for (XMLDifference outList : diffReportMap) {
				// System.out.println("SI No. : " + count++);
				// System.out.println("Source Node : [" +
				// outList.getSourceNode() + "] Value : ["
				// + outList.getSourceNodeValue() + "]");
				// System.out.println("Target Node : [" +
				// outList.getTargetNode() + "] Value : ["
				// + outList.getTargetNodeValue() + "]");
				// System.out.println("Difference Description : " +
				// outList.getDescription());
				// System.out.println("Node Path : " + outList.getIgnorePath());
				//
				// System.out.println("-----------------------------------------------");
				// }
				//
				// if (outValList.size() == 0) {
				// System.out.println("Success : The Source and Target XML's are
				// Identical.");
				// } else {
				// System.out.println("Failure : The Source and Target XML's are
				// not Identical, please refer the details report.");
				// }
				System.out.println("************ Processing XML Comparison ***********");
				DateFormat dateFormat = new SimpleDateFormat("yyyy_MM_dd_HHmmss");
				Date date = new Date();
				reportPath = createDirectory(reportPath, null) + "\\" + serviceName + "_Difference_Report_"
						+ dateFormat.format(date) + ".html";
				generateReport(reportPath, diffReportMap, serviceName, sourceXML, targetXML);
				System.out.println("Report Generated Successfully in location  : " + reportPath);
				System.out.println("************ End Of Process ***********");
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

		return diffReportMap;

	}

	@SuppressWarnings("unused")
	private static void generateReport(String fileNameWithPath, List<XMLDifference> reportModelList, String serviceName,
			String sourceXML, String targetXML) {
		try {
			File data = new File(fileNameWithPath);
			FileWriter writer = new FileWriter(data);
			StringBuffer strOut = new StringBuffer();
			int siNo = 0;
			String outMessage = null;

			int sourceTagCount = getTagCount(sourceXML);
			int targetTagCount = getTagCount(targetXML);

			if (reportModelList.size() == 0) {
				outMessage = "Success : The Source and Target XML's are Identical.";
			} else {
				outMessage = "Failure : The Source and Target XML's are not Identical. Please check the below report for more details.";
			}
			strOut.append("<table style='width: 100%; background-color: #ddd;' border='1'><caption>&nbsp;</caption>\n");
			strOut.append("<tbody>\n");
			strOut.append("<tr>\n");
			strOut.append("<td style='background-color: #e20074; text-align: center;' colspan='7'>\n");
			strOut.append("<h1><span style='color: #ffffff;'><strong>XML Comparison Report</strong></span></h1>\n");
			strOut.append("</td>\n");
			strOut.append("</tr>\n");
			strOut.append("<tr>\n");
			strOut.append("<td colspan='2'><strong>Service/Operation Name</strong></td>\n");
			strOut.append("<td colspan='5'>" + serviceName + "</td>\n");
			strOut.append("</tr>\n");
			strOut.append("<tr>\n");
			strOut.append("<td colspan='2'><strong>Legacy Response File Path</strong></td>\n");
			strOut.append("<td colspan='5'>" + sourceXML + "</td>\n");
			strOut.append("</tr>\n");
			strOut.append("<tr>\n");
			strOut.append("<td colspan='2'><strong>Migrated Response File Path</strong></td>\n");
			strOut.append("<td colspan='5'>" + targetXML + "</td>\n");
			strOut.append("</tr>\n");
			strOut.append("<tr>\n");
			strOut.append("<td colspan='2'><strong>Legacy Response Tag Count</strong></td>\n");
			strOut.append("<td colspan='5'>" + sourceTagCount + "</td>\n");
			strOut.append("</tr>\n");
			strOut.append("<tr>\n");
			strOut.append("<td colspan='2'><strong>Migrated Response Tag Count</strong></td>\n");
			strOut.append("<td colspan='5'>" + targetTagCount + "</td>\n");
			strOut.append("</tr>\n");
			strOut.append("<tr>\n");
			strOut.append("<td colspan='2'><strong>Comparison Result</strong></td>\n");
			strOut.append("<td colspan='5'>" + outMessage + "</td>\n");
			strOut.append("</tr>\n");
			strOut.append("<tr>\n");
			strOut.append("<table style='width: 100%; background-color: #ddd;' border='1'><caption>&nbsp;</caption>\n");
			strOut.append("<tbody>\n");
			strOut.append(
					"<tr><th style='background-color: #e20074; text-align: center; color: #ffffff;'>Sl. No.</th><th style='background-color: #e20074; color: #ffffff; text-align: center;'>Description</th><th style='background-color: #e20074; color: #ffffff; text-align: center;'>Legacy Response Node</th><th style='background-color: #e20074; color: #ffffff; text-align: center;'>Legacy Response Node Value</th><th style='background-color: #e20074; color: #ffffff; text-align: center;'>Migrated Response Node</th><th style='background-color: #e20074; color: #ffffff; text-align: center;'>Migrated Response Node Value</th><th style='background-color: #e20074; color: #ffffff; text-align: center;'>Node Path</th></tr>\n");
			if (reportModelList == null) {
				strOut.append("<tr>\n");
				strOut.append(
						"<td colspan='7' style='text-align: center; word-wrap: break-word; word-break: break-all;'>*** No Data Found ***</td>\n");
				strOut.append("</tr>\n");
			} else {
				for (XMLDifference v : reportModelList) {

					++siNo;

					strOut.append("<tr>\n");
					strOut.append(
							"<td style='width: 4%; text-align: center; word-wrap: break-word; word-break: break-all;'>"
									+ siNo + "</td>\n");
					strOut.append("<td style='width: 18%; word-wrap: break-word; word-break: break-all;'>"
							+ v.getDescription() + "</td>\n");
					strOut.append("<td style='width: 10%; word-wrap: break-word; word-break: break-all;'>"
							+ v.getSourceNode() + "</td>\n");
					strOut.append("<td style='width: 10%; word-wrap: break-word; word-break: break-all;'>"
							+ v.getSourceNodeValue() + "</td>\n");
					strOut.append("<td style='width: 10%; word-wrap: break-word; word-break: break-all;'>"
							+ v.getTargetNode() + "</td>\n");
					strOut.append("<td style='width: 10%; word-wrap: break-word; word-break: break-all;'>"
							+ v.getTargetNodeValue() + "</td>\n");
					strOut.append("<td style='width: 30%; word-wrap: break-word; word-break: break-all;'>"
							+ v.getIgnorePath() + "</td>\n");
					strOut.append("</tr>\n");

				}
				if (reportModelList.size() == 0) {
					strOut.append("<tr>\n");
					strOut.append(
							"<td colspan='7' style='text-align: center; word-wrap: break-word; word-break: break-all;'>*** No Data Found ***</td>\n");
					strOut.append("</tr>\n");
				}
			}
			strOut.append("</tbody>\n");
			strOut.append("</table>\n");
			strOut.append("</tr>\n");
			strOut.append("</tbody>\n");
			strOut.append("</table>\n");
			String clobStr = strOut.toString();

			writer.write(clobStr);
			writer.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

}
